/*
 * tmr_regs.h --
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2012, Maxim Integrated Products
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the <organization> nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MAXIM INTEGRATED PRODUCTS ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* [INTERNAL] ------------------------------------------------------------------
 * Created on: June 5, 2012
 * Author: Jeremy Brodt
 *
 * ---- Subversion keywords (need to set the keyword property)
 * $Revision:: 21       $:  Revision of last commit
 * $Author:: ray.vandew#$:  Author of last commit
 * $Date:: 2013-06-07 1#$:  Date of last commit
 * [/INTERNAL] -------------------------------------------------------------- */

#ifndef TMR_REGS_H_
#define TMR_REGS_H_

/** @file tmr_regs.h TMR Registers Header */

/** @defgroup TMR TMR Driver
 * @ingroup DRIVER
 */

/** @defgroup TMR_REGS TMR Registers
 *
 * The timer can be used for timing, event counting, or generation of
 * pulse-width modulated (PWM) signals. The timers’ features include:
 * @li 32-bit reload counter
 * @li Timer output pin
 * @li Programmable prescaler with prescale values from 1 to 4096
 * @li PWM output generation
 * @li Capture, compare and capture/compare capability
 * @li External input pin for timer input, clock gating, or capture signal
 * @li Timer interrupt
 *
 * @see Zatara® ASSP (ZA9L2) Functional Design Specification

 *
 * @ingroup TMR
 * @{
 */


/** @defgroup TMR_REGS_CNT Count Register
 *
 * In Counter mode, the Timer register, stores the 32-bit Count value. Writing
 * to the Timer register while the timer is enabled is not recommended. There
 * are no temporary holding registers available for write operations, so
 * simultaneous 32-bit writes are not possible. If the Timer register is
 * written during counting, the 32-bit written value is placed in the counter
 * at the next clock edge. The counter continues counting from the new value.
 *
 * @{
 */
#define TMR_CNT_OFST	0x00 //!< Register Offset

/** @} */ /* @defgroup TMR_REGS_CNT */

/** @defgroup TMR_REGS_CMP Compare Register
 *
 * In Compare and Capture/Compare modes, the Timer Compare register stores the
 * 32-bit Compare value.
 *
 * @{
 */
#define TMR_CMP_OFST	0x04 //!< Register Offset

/** @} */ /* @defgroup TMR_REGS_CMP */

/** @defgroup TMR_REGS_PWM PWM Register
 *
 * PWM forms a 32-bit value that is compared to the current
 * 32-bit timer count. When a match occurs, the PWM output changes state. The
 * PWM output value is set by the TPOL bit in the Timer Control Register
 * (TMR_REGS_CTRL). In Capture and Capture/Compare modes, the PWM
 * register stores the 32-bit captured timer value.

 *
 * @li
 *
 * @{
 */
#define TMR_PWM_OFST	0x08 //!< Register Offset

/** @} */ /* @defgroup TMR_REGS_PWM */

/** @defgroup TMR_REGS_INT Interrupt Register
 *
 * Register for interruption clearing.
 *
 * @{
 */
#define TMR_INT_OFST	0x0C //!< Register Offset

#define TMR_INT_CLR_MASK 0x1

/** @} */ /* @defgroup TMR_REGS_INT */

/** @defgroup TMR_REGS_CTRL Controller Register
 *
 * The Timer Control register enables/disables the timers, sets the prescaler
 * value, and determines the timer operating mode.
 *
 * @li TMODE [0:2]: Timer Mode
 * @li PRES [5:3]: Prescale Value
 * @li TPOL [6]: Timer Input/Output Polarity
 * @li TEN [7]: Timer Enable
 *
 * @{
 */
#define TMR_CTRL_OFST	0x10 //!< Register Offset

#define TMR_CTRL_TMODE_OFST		0	//!< Timer Mode offset
#define TMR_CTRL_TMODE_MASK		0x7	//!< Timer Mode mask
#define TMR_CTRL_TMODE_ONE_SHOT	0	//!< One shot mode
#define TMR_CTRL_TMODE_CONTINUOUS	1	//!< Continuous mode
#define TMR_CTRL_TMODE_COUNTER	2	//!< Counter mode
#define TMR_CTRL_TMODE_PWM		3	//!< PWM mode
#define TMR_CTRL_TMODE_CAPTURE	4	//!< Capture mode
#define TMR_CTRL_TMODE_COMPARE	5	//!< Compare mode
#define TMR_CTRL_TMODE_GATED		6	//!< Gated mode
#define TMR_CTRL_TMODE_CAPCOMP	7	//!< Capture/Compare mode

#define TMR_CTRL_PRES_OFST	3		//!< Prescale offset
#define TMR_CTRL_PRES_MASK	0x27	//!< Prescale make
#define TMR_CTRL_PRES_DIV1	0		//!< Divide by 1
#define TMR_CTRL_PRES_DIV2	1		//!< Divide by 2
#define TMR_CTRL_PRES_DIV4	2		//!< Divide by 4
#define TMR_CTRL_PRES_DIV8	3		//!< Divide by 8
#define TMR_CTRL_PRES_DIV16	4		//!< Divide by 16
#define TMR_CTRL_PRES_DIV32	5		//!< Divide by 32
#define TMR_CTRL_PRES_DIV64	6		//!< Divide by 64
#define TMR_CTRL_PRES_DIV128	7		//!< Divide by 128
#define TMR_CTRL_PRES_DIV256	0x20	//!< Divide by 256
#define TMR_CTRL_PRES_DIV512	0x21	//!< Divide by 512
#define TMR_CTRL_PRES_DIV1024	0x22	//!< Divide by 1024
#define TMR_CTRL_PRES_DIV2048	0x23	//!< Divide by 2048
#define TMR_CTRL_PRES_DIV4096	0x24	//!< Divide by 4096


#define TMR_CTRL_TPOL_OFST	6	//!< Timer Input/Output Polarity offset
#define TMR_CTRL_TPOL_MASK	0x1	//!< Timer Input/Output Polarity mask
#define TMR_CTRL_TPOL_HIGH	1	//!< Polarity high
#define TMR_CTRL_TPOL_LOW		0	//!< Polarity low

#define TMR_CTRL_TEN_OFST		7	//!< Timer enable offset
#define TMR_CTRL_TEN_MASK		0x1	//!< Timer enable mask
#define TMR_CTRL_TEN_ENABLE	1	//!< Enable timer

#define	TMR_RSTR_OFST			16	//!< Offset for Timer reset [18:16] PWMx reset
#define	TMR_ACTIVE_OFST		16 	//!< offset for clock enable [18:16] PWMx: PWM/timers

/** @} */ /* @defgroup TMR_REGS_CTRL */

#ifndef __ASSEMBLER__
/* -------------------------------------------------------------------------- */
/** TMR Registers.
 *
 */
typedef volatile struct _tmr_regs_ {
	unsigned int count; //!< Count register
	unsigned int compare; //!< Compare register
	unsigned int pwm; //!< PWM register
	unsigned int interrupt; //!< Interrupt register
	unsigned int control; //!< Control register
} tmr_regs_t;

/** Timer Prescale Values
 *
 */
typedef enum tmr_prescale_ {
	TMR_PRES_DIV_1= 0,	//!< Prescale: divide by 1
	TMR_PRES_DIV_2,		//!< Prescale: divide by 2
	TMR_PRES_DIV_4,		//!< Prescale: divide by 4
	TMR_PRES_DIV_8,		//!< Prescale: divide by 8
	TMR_PRES_DIV_16,		//!< Prescale: divide by 16
	TMR_PRES_DIV_32,		//!< Prescale: divide by 32
	TMR_PRES_DIV_64,		//!< Prescale: divide by 64
	TMR_PRES_DIV_128,		//!< Prescale: divide by 128
	TMR_PRES_DIV_256,		//!< Prescale: divide by 256
	TMR_PRES_DIV_512,		//!< Prescale: divide by 512
	TMR_PRES_DIV_1024,	//!< Prescale: divide by 1024
	TMR_PRES_DIV_2048,	//!< Prescale: divide by 2048
	TMR_PRES_DIV_4096,	//!< Prescale: divide by 4096
} tmr_prescale_t;

/** Timer Modes
 *
 */
typedef enum tmr_mode_ {
	TMR_MODE_ONE_SHOT	= 0,//!< One shot mode
	TMR_MODE_CONTINUOUS,	//!< Continuous mode
	TMR_MODE_COUNTER,		//!< Counter mode
	TMR_MODE_PWM,			//!< PWM mode
	TMR_MODE_CAPTURE,		//!< Capture mode
	TMR_MODE_COMPARE,		//!< Compare mode
	TMR_MODE_GATED,		//!< Gated mode
	TMR_MODE_CAPCOMP,		//!< Capture/Compare mode
} tmr_mode_t;

#endif /* __ASSEMBLER__ */

/** @} *//* @defgroup TMR_REGS */

#endif /* TMR_REGS_H_ */
