/*
 * uart_regs.h --
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2012, Maxim Integrated Products
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the <organization> nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MAXIM INTEGRATED PRODUCTS ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* [INTERNAL] ------------------------------------------------------------------
 * Created on: June 5, 2012
 * Author: Jeremy Brodt
 *
 * ---- Subversion keywords (need to set the keyword property)
 * $Revision: 21 $:  Revision of last commit
 * $Author: ray.vandewalker $:  Author of last commit
 * $Date: 2013-06-07 16:34:10 -0700 (Fri, 07 Jun 2013) $:  Date of last commit
 * [/INTERNAL] -------------------------------------------------------------- */

#ifndef UART_REGS_H_
#define UART_REGS_H_

/** @file uart_regs.h UART Registers Header */

/** @defgroup UART UART Driver
 * @ingroup DRIVER
 */

/** @defgroup UART_REGS UART Registers
 *
 * @note UART IP or Specification version number
 *
 * @ingroup UART
 * @{
 */

/** @defgroup UART_REGS_CR Control Register (CR)
 *
 * @li RXTHD [3:0]	- R/W: receive FIFO threshold
 * @li PAREN [4] 	- R/W: parity enable
 * @li PAREO [5]	- R/W: parity mode (odd or even)
 * @li PARMD [6]	- R/W: parity mode
 * @li RFU1 [7]		- R: reserved
 * @li TXFLUSH [8]	- R/W: flush transmit FIFO, clear by hw
 * @li RXFLUSH [9]	- R/W: flush receive FIFO , clear by hw
 * @li SIZE [11:10] - R/W: data format, number of bits per data
 * @li STOP [12]	- R/W: number of stop bits
 * @li RTSCTS [13]	- R/W: hardware flow control enable
 * @li RFU2 [31:14]	- R: reserved
 *
 * @{
 */

#define UART_CR_OFST	0x0	//!< Register offset
#define UART_CR_DFLT	0x0	//!< Register default value

/* Bits Fields */

#define UART_CR_RXTHD_OFST		0	//!< RXTHD offset
#define UART_CR_PAREN_OFST		4	//!< PAREN offset
#define UART_CR_PAREO_OFST		5	//!< PAREO offset
#define UART_CR_PARMD_OFST		6	//!< PARMD offset
#define UART_CR_RFU1_OFST		7	//!< RFU1 offset
#define UART_CR_TXFLUSH_OFST	8	//!< TXFLUSH offset
#define UART_CR_RXFLUSH_OFST	9	//!< RXFLUSH offset
#define UART_CR_SIZE_OFST		10	//!< SIZE offset
#define UART_CR_STOP_OFST		12	//!< STOP	offset
#define UART_CR_RTSCTS_OFST		13	//!< CTSRTS offset
#define UART_CR_RFU2_OFST		14	//!< RFU2 offset


#define UART_CR_RXTHD_MASK		0x7	//!< RXTHD mask
#define UART_CR_PAREN_MASK		0x10//!< PAREN mask
#define UART_CR_PAREO_MASK		0x20//!< PAREO mask
#define UART_CR_PARMD_MASK		0x40//!< PARMD mask
#define UART_CR_RFU1_MASK		0x80//!< RFU1 mask
#define UART_CR_TXFLUSH_MASK	0x100//!< TXFLUSH mask
#define UART_CR_RXFLUSH_MASK	0x200//!< RXFLUSH mask
#define UART_CR_SIZE_MASK		0xC00//!< SIZE mask
#define UART_CR_STOP_MASK		0x1000//!< STOP	mask
#define UART_CR_RTSCTS_MASK	    0x2000//!< CTSRTS mask
#define UART_CR_RFU2_MASK		0xFFFFc000//!< RFU2 mask


/* Enumeration of the different value for a bits field . We recommend to use
 * human readable value. */

#define UART_CR_PAREN_DISABLE	0	//!< no parity bit
#define UART_CR_PAREN_ENABLE	1	//!< one parity bit

#define UART_CR_PAREO_ODD		1	//!< odd parity
#define UART_CR_PAREO_EVEN		0	//!< even parity

#define UART_CR_PARMD_O			1	//!< parity based on the number of 0s
#define UART_CR_PARMD_1			0	//!< parity based on the number of 1s

#define UART_CR_TXFLUSH_FLUSH	1	//!< flush TX FIFO

#define UART_CR_RXFLUSH_FLUSH	1	//!< flush RX FIFO

#define UART_CR_SIZE_5			0	//!< 5-bit data format
#define UART_CR_SIZE_6			1	//!< 6-bit data format
#define UART_CR_SIZE_7			2	//!< 7-bit data format
#define UART_CR_SIZE_8			3	//!< 8-bit data format

#define UART_CR_RTSCTS_DISABLE	0	//!< hardware flow control disable
#define UART_CR_RTSCTS_ENABLE	1	//!< hardware flow control enable

/** @} */ /* @defgroup UART_REGS_CR */


/** @defgroup UART_REGS_SR Status Register (SR)
 *
 * @li TXBUSY [0]	- R: UART is transmitting data
 * @li RXBUSY [1] 	- R: UART is receiving data
 * @li RFU1 [2]		- R: reserved
 * @li RXEMPTY [6]	- R: RX FIFO empty flag
 * @li RXFULL [6]	- R: TX FIFO full flag
 * @li TXEMPTY [7]	- R: TX FIFO empty flag
 * @li TXFULL [6]	- R: TX FIFO full flag
 * @li RXELT [7]	- R: number of element in RX FIFO
 * @li TXELT [6]	- R: number of element in TX FIFO
 * @li RFU2 [31:16]	- R: reserved
 *
 * @{
 */

#define UART_SR_OFST	0x4	//!< Register offset
#define UART_SR_DFLT	0x50//!< Register default value

/* Bits Fields */

#define UART_SR_TXBUSY_OFST		0	//!< TXBUSY offset
#define UART_SR_RXBUSY_OFST		1	//!< RXBUSY offset
#define UART_SR_RFU1_OFST		2	//!< RFU1 offset
#define UART_SR_RXEMPTY_OFST	4	//!< RXEMPTY offset
#define UART_SR_RXFULL_OFST		5	//!< RXFULL offset
#define UART_SR_TXEMPTY_OFST	6	//!< TXEMPTY offset
#define UART_SR_TXFULL_OFST		7	//!< TXFULL offset
#define UART_SR_RXELT_OFST		8	//!< RXELT offset
#define UART_SR_TXELT_OFST		12	//!< TXELT offset
#define UART_SR_RFU2_OFST		16	//!< RFU2 offset

#define UART_SR_TXBUSY_MASK		0x1	//!< TXBUSY mask
#define UART_SR_RXBUSY_MASK		0x2	//!< RXBUSY mask
#define UART_SR_RFU1_MASK		0xC	//!< RFU1 mask
#define UART_SR_RXEMPTY_MASK	0x10//!< RXEMPTY mask
#define UART_SR_RXFULL_MASK		0x20//!< RXFULL mask
#define UART_SR_TXEMPTY_MASK	0x40//!< TXEMPTY mask
#define UART_SR_TXFULL_MASK		0x80//!< TXFULL mask
#define UART_SR_RXELT_MASK		0xF00//!< RXELT mask
#define UART_SR_TXELT_MASK		0xF000//!< TXELT mask
#define UART_SR_RFU2_MASK		0xFFFF0000//!< RFU2 mask

#define UART_SR_TXBUSY_BUSY		1	//!< UART is transmitting data
#define UART_SR_RXBUSY_BUSY		1	//!< UART is receiving data
#define UART_SR_RXEMPTY_EMPTY	1	//!< RX FIFO is empty
#define UART_SR_RXFULL_FULL		1	//!< RX FIFO is full
#define UART_SR_TXEMPTY_EMPTY	1	//!< TX FIFO is empty
#define UART_SR_TXFULL_FULL		1	//!< TX FIFO is full

#define UART_STATUS_TX_BUSY		(0x01 << 0) /** UART is transmitting data */
#define UART_STATUS_RX_BUSY		(0x01 << 1) /** UART is receiving data */
#define UART_STATUS_RX_EMPTY	(0x01 << 4) /** RX FIFO is empty */
#define UART_STATUS_RX_FULL		(0x01 << 5) /** RX FIFO is full */
#define UART_STATUS_TX_EMPTY	(0x01 << 6) /** TX FIFO is empty */
#define UART_STATUS_TX_FULL		(0x01 << 7) /** TX FIFO is full */
#define UART_DATA_RX_PARITY		(0x01 << 8) /** parity error occurred during the byte reception */

#define	UART_BRR_DIV_FACTOR		128
#define	UART_RCR_DDIV_FACTOR	128

#define UART_BRR0(baudrate)   (PCLK_HZ / (baudrate << 7)) /** Find BRR0 */

#define UART_BRR1(baudrate)   \
    ((PCLK_HZ / baudrate) - (UART_BRR0(baudrate) << 7)) /** Find BRR1 */

#define UART_BAUD_RATE_MASK_IDIV	0x00000fff
#define UART_BAUD_RATE_MASK_DDIV	0x0000007f
#define UART_CTL_TRANSFER_SIZE_MASK	(0x03 << UART_CR_SIZE_OFST)

#define	UART0_PAD_CFG_OFFSET		16
#define	UART1_PAD_CFG_OFFSET		24
#define	UART_PAD_CFG_MASK			0x0f

/** @} */ /* @defgroup UART_REGS_SR */


/** @defgroup UART_REGS_IER Interrupt Enable Register (IER)
 *
 * @li FRAMIE [0]	- R/W: framing error interrupt flag
 * @li PARIE [1] 	- R/W: parity error interrupt flag
 * @li SIGIE [2]	- R/W: CTS line toggling interrupt flag
 * @li OVERIE [3]	- R/W: overrun on receiving interrupt flag
 * @li FFRXIE [4]	- R/W: RX FIFO threshold interrupt flag
 * @li FFTXOIE [5]	- R/W: TX FIFO last byte interrupt flag
 * @li FFTXHIE [6]	- R/W: TX FIFO half level interrupt flag
 * @li RFU [31:7]	- R: reserved
 *
 * @{
 */

#define UART_IER_OFST	0x8	//!< Register offset
#define UART_IER_DFLT	0x0 //!< Register default value

/* Bits Fields */

#define UART_IER_FRAMIE_OFST	0	//!< FRAMIE offset
#define UART_IER_PARIE_OFST		1	//!< PARIE offset
#define UART_IER_SIGIE_OFST		2	//!< SIGIE offset
#define UART_IER_OVERIE_OFST	3	//!< OVERIE offset
#define UART_IER_FFRXIE_OFST	4	//!< FFRXIE offset
#define UART_IER_FFTXOIE_OFST	5	//!< FFTXOIE offset
#define UART_IER_FFTXHIE_OFST	6	//!< FFTXHIE offset
#define UART_IER_RFU_OFST		7	//!< RFU offset

#define UART_IER_FRAMIE_MASK	0x1	//!< FRAMIE mask
#define UART_IER_PARIE_MASK		0x2	//!< PARIE mask
#define UART_IER_SIGIE_MASK		0x4	//!< SIGIE mask
#define UART_IER_OVERIE_MASK	0x8	//!< OVERIE mask
#define UART_IER_FFRXIE_MASK	0x10//!< FFRXIE mask
#define UART_IER_FFTXOIE_MASK	0x20//!< FFTXOIE mask
#define UART_IER_FFTXHIE_MASK	0x40//!< FFTXHIE mask
#define UART_IER_RFU_MASK		0xFFFFFF80//!< RFU mask

#define UART_IER_FRAMIE_DISABLE	0	//!< framing error interrupt disable
#define UART_IER_FRAMIE_ENABLE	1	//!< framing error interrupt enable

#define UART_IER_PARIE_DISABLE	0	//!< parity error interrupt disable
#define UART_IER_PARIE_ENABLE	1	//!< parity error interrupt enable

#define UART_IER_SIGIE_DISABlE	0	//!< CTS changed interrupt disable
#define UART_IER_SIGIE_ENABlE	1	//!< CTS changed interrupt enable

#define UART_IER_OVERIE_DISABLE	0	//!< overrun interrupt disable
#define UART_IER_OVERIE_ENABLE	1	//!< overrun interrupt enable

#define UART_IER_FFRXIE_DISABLE	0	//!< RX FIFO threshold interrupt disable
#define UART_IER_FFRXIE_ENABLE	1	//!< RX FIFO threshold interrupt enable

#define UART_IER_FFTXOIE_DISABLE    0	//!< TX FIFO last byte interrupt disable
#define UART_IER_FFTXOIE_ENABLE	1	//!< TX FIFO last byte interrupt enable

#define UART_IER_FFTXHIE_DISABLE	0	//!< TX FIFO half filled interrupt disable
#define UART_IER_FFTXHIE_ENABLE	1	//!< TX FIFO half filled interrupt enable

/** @} */ /* @defgroup UART_REGS_IER */


/** @defgroup UART_REGS_ISR Interrupt Status Register (ISR)
 *
 * @li FRAMIS [0]	- R/C: framing error interrupt flag
 * @li PARIS [1] 	- R/C: parity error interrupt flag
 * @li SIGIS [2]	- R/C: CTS line toggling interrupt flag
 * @li OVERIS [3]	- R/C: overrun on receiving interrupt flag
 * @li FFRXIS [4]	- R/C: RX FIFO threshold interrupt flag
 * @li FFTXOIS [5]	- R/C: TX FIFO last byte interrupt flag
 * @li FFTXHIS [6]	- R/C: TX FIFO half level interrupt flag
 * @li RFU [31:7]	- R: reserved
 *
 * @{
 */

#define UART_ISR_OFST	0xC	//!< Register offset
#define UART_ISR_DFLT	0x0//!< Register default value

/* Bits Fields */

#define UART_ISR_FRAMIS_OFST	0	//!< FRAMIS offset
#define UART_ISR_PARIS_OFST		1	//!< PARIS offset
#define UART_ISR_SIGIS_OFST		2	//!< SIGIS offset
#define UART_ISR_OVERIS_OFST	3	//!< OVERIS offset
#define UART_ISR_FFRXIS_OFST	4	//!< FFRXIS offset
#define UART_ISR_FFTXOIS_OFST	5	//!< FFTXOIS offset
#define UART_ISR_FFTXHIS_OFST	6	//!< FFTXHIS offset
#define UART_ISR_RFU_OFST		7	//!< RFU offset

#define UART_ISR_FRAMIS_MASK	0x1	//!< FRAMIS mask
#define UART_ISR_PARIS_MASK		0x2	//!< PARIS mask
#define UART_ISR_SIGIS_MASK		0x4	//!< SIGIS mask
#define UART_ISR_OVERIS_MASK	0x8	//!< OVERIS mask
#define UART_ISR_FFRXIS_MASK	0x10//!< FFRXIS mask
#define UART_ISR_FFTXOIS_MASK	0x20//!< FFTXOIS mask
#define UART_ISR_FFTXHIS_MASK	0x40//!< FFTXHIS mask
#define UART_ISR_RFU_MASK		0xFFFFFF80//!< RFU mask

#define UART_ISR_FRAMIS_ERROR	1	//!< framing error
#define UART_ISR_PARIS_ERROR	1	//!< parity error
#define UART_ISR_SIGIS_TOGGLING	1	//!< CTS changed
#define UART_ISR_OVERIS_ERROR	1	//!< OVERIS error
#define UART_ISR_FFRXIS_THRESHOLD	1	//!< RX FIFO threshold reached
#define UART_ISR_FFTXOIS_ONE	1	//!< TX FIFO last byte
#define UART_ISR_FFTXHIS_HALF	1	//!< TX FIFO half filled

/** @} */ /* @defgroup UART_REGS_ISR */


/** @defgroup UART_REGS_BRR0 Baud Rate Register 0 (BRR0)
 *
 * @li IDIV [11:0]	- R/W: integer divider
 * @li RFU [31:12]	- R: reserved
 *
 * @{
 */

#define UART_BRR_OFST		0x10	//!< Register offset
#define UART_BRR_DFLT		0x0		//!< Register default value

/* Bits Fields */

#define UART_BRR_IDIV_OFST		0	//!< IDIV offset
#define UART_BRR_RFU_OFST		12	//!< RFU offset

#define UART_BRR_IDIV_MASK		0x7FF	//!< IDIV mask
#define UART_BRR_RFU_MASK		0xFFFFF800//!< RFU mask

/** @} */ /* @defgroup UART_REGS_BRR0 */


/** @defgroup UART_REGS_BBR1 Baud Rate Register 1 (BRR1)
 *
 * @li DDIV [6:0]	- R/W: decimal divider
 * @li RFU [31:7]	- R: reserved
 *
 * @{
 */

#define UART_BRR1_OFST			0x14	//!< Register offset
#define UART_BRR1_DFLT			0x0		//!< Register default value

/* Bits Fields */

#define UART_BRR1_IDIV_OFST	    0	//!< DDIV offset
#define UART_BRR1_RFU_OFST		7	//!< RFU offset

#define UART_BRR1_IDIV_MASK	    0x3F//!< DDIV mask
#define UART_BRR1_RFU_MASK		0xFFFFFFC0//!< RFU mask

/** @} */ /* @defgroup UART_REGS_BRR1 */


/** @defgroup UART_REGS_TXR Transmit Register (TXR)
 *
 * @li DATA [7:0]	- R/W: TX data read out (for debug purpose)
 * @li RFU [31:8]	- R: reserved
 *
 * @{
 */

#define UART_TXR_OFST		0x18	//!< Register offset
#define UART_TXR_DFLT		0x0		//!< Register default value

/* Bits Fields */

#define UART_TXR_DATA_OFST		0	//!< DATA offset
#define UART_TXR_RFU_OFST		8	//!< RFU offset

#define UART_TXR_DATA_MASK		0xFF//!< DATA mask
#define UART_TXR_RFU_MASK		0xFFFFFF00//!< RFU mask

/** @} */ /* @defgroup UART_REGS_TXR */


/** @defgroup UART_REGS_PR Pins Register (PR)
 *
 * @li CTS [0]		- R: CTS pin status
 * @li CTS [1]		- R/W: RTS pin control
 * @li RFU [31:2]	- R: reserved
 *
 * @{
 */

#define UART_PR_OFST			0x1C	//!< Register offset
#define UART_PR_DFLT			0x3		//!< Register default value

/* Bits Fields */

#define UART_PR_CTS_OFST		0	//!< CTS offset
#define UART_PR_RTS_OFST		1	//!< RTS offset
#define UART_PR_RFU_OFST		2	//!< RFU offset

#define UART_PR_CTS_MASK		0x1//!< CTS mask
#define UART_PR_RTS_MASK		0x2//!< RTS mask
#define UART_PR_RFU_MASK		0xFFFFFFFC//!< RFU mask

/** @} */ /* @defgroup UART_REGS_PR */


/** @defgroup UART_REGS_DR Data Register (DR)
 *
 * @li DATA [7:0]	- R/W: RX/TX data
 * @li PARITY [8]	- R/W: RX data parity
 * @li RFU [31:9]	- R: reserved
 *
 * @{
 */

#define UART_DR_OFST	0x20	//!< Register offset
#define UART_DR_DFLT	0x0		//!< Register default value

/* Bits Fields */

#define UART_DR_DATA_OFST		0	//!< DATA offset
#define UART_DR_PARITY_OFST	    8	//!< PARITY offset
#define UART_DR_RFU_OFST		9	//!< RFU offset

#define UART_DR_DATA_MASK		0xFF//!< DATA mask
#define UART_DR_PARITY_MASK	    0x100//!< PARITY mask
#define UART_DR_RFU_MASK		0xFFFFFE00//!< RFU mask

/** @} */ /* @defgroup UART_REGS_DR */


#ifndef __ASSEMBLER__
/* -------------------------------------------------------------------------- */
/** UART Registers.
 *
 */
typedef volatile struct _uart_regs_{
	volatile unsigned cr;	//< control register
	volatile unsigned sr;	//< status register
	volatile unsigned ier;	//< interrupt enable register
	volatile unsigned isr;	//< interrupt status register
	volatile unsigned brr;	//< baudrate register 0
	volatile unsigned rcr;	//< baudrate register 1
	volatile unsigned txr;	//< TX register
	volatile unsigned pnr;	//< pin register
	volatile unsigned dr;	//< data register
} uart_regs_t;

#endif /* __ASSEMBLER__ */

/** @} */ /* @defgroup UART_REGS */

#endif /* UART_REGS_H_ */
