/* -----------------------------------------------------------------------------
 * Copyright (c) 2014 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *   
 *
 * $Date:        15. April 2014
 * $Revision:    V1.00
 *  
 * Project:      Flash Programming Functions for Maxim MAX716xx Flash
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.00
 *    Initial release
 */ 

#include "..\FlashOS.H"        // FlashOS Structures

typedef volatile unsigned char    vu8;
typedef          unsigned char     u8;
typedef volatile unsigned short   vu16;
typedef          unsigned short    u16;
typedef volatile unsigned long    vu32;
typedef          unsigned long     u32;

#define M8(adr)  (*((vu8  *) (adr)))
#define M16(adr) (*((vu16 *) (adr)))
#define M32(adr) (*((vu32 *) (adr)))

// Peripheral Memory Map
#define FLASH_BASE        0x40061000

// Flash Registers
typedef struct {
  vu32 FLSH_ADDR;
  vu32 FLSH_CLKDIV;
  vu32 FLSH_CN;
  vu32 PR1E_ADDR;
  vu32 PR2S_ADDR;
  vu32 PR2E_ADDR;
  vu32 PR3S_ADDR;
  vu32 PR3E_ADDR;
  vu32 FLSH_MD;
  vu32 FLSH_INT;
	vu32 RESERVED0;
	vu32 RESERVED1;
  vu32 FLSH_DATA0;
  vu32 FLSH_DATA1;
  vu32 FLSH_DATA2;
  vu32 FLSH_DATA3;
} FLASH_TypeDef;

#define FLASH           ((FLASH_TypeDef*) FLASH_BASE)

#define FLSH_CN_PEND    0x01000000
#define FLSH_INT_AF     0x00000002

#define FLSH_CN_UNLOCK_MASK     0xF0000000
#define FLSH_CN_UNLOCK_VALUE    0x20000000

// SHOULD_BE_CHECKED????
#define FLSH_CLKDIV_VAL         (140)
#define FLSH_SECTOR_SZ          (8192)

vu32 flsh_cn;
vu32 flsh_int;

void maxim_flash_op_pre(void)
{
  /* Refresh flash controller timing */
  FLASH->FLSH_CLKDIV = FLSH_CLKDIV_VAL;
  
  /* Clear and disable flash programming interrupts */
  flsh_int        = FLASH->FLSH_INT;
  FLASH->FLSH_INT = 0x00000000;
  
  /* Unlock flash */
  flsh_cn  = FLASH->FLSH_CN;
  flsh_cn &= ~FLSH_CN_UNLOCK_MASK;
  flsh_cn |= FLSH_CN_UNLOCK_VALUE;
  FLASH->FLSH_CN = flsh_cn;
}

void maxim_flash_op_post(void)
{
  /* Lock flash */
  flsh_cn  = FLASH->FLSH_CN;
  flsh_cn &= ~FLSH_CN_UNLOCK_MASK;
  FLASH->FLSH_CN = flsh_cn;
  
  /* Restore flash programming interrupts */
  FLASH->FLSH_INT = flsh_int;
}


/* 
   Mandatory Flash Programming Functions (Called by FlashOS):
                int Init        (unsigned long adr,   // Initialize Flash
                                 unsigned long clk,
                                 unsigned long fnc);
                int UnInit      (unsigned long fnc);  // De-initialize Flash
                int EraseSector (unsigned long adr);  // Erase Sector Function
                int ProgramPage (unsigned long adr,   // Program Page Function
                                 unsigned long sz,
                                 unsigned char *buf);

   Optional  Flash Programming Functions (Called by FlashOS):
                int BlankCheck  (unsigned long adr,   // Blank Check
                                 unsigned long sz,
                                 unsigned char pat);
                int EraseChip   (void);               // Erase complete Device
      unsigned long Verify      (unsigned long adr,   // Verify Function
                                 unsigned long sz,
                                 unsigned char *buf);

       - BlanckCheck  is necessary if Flash space is not mapped into CPU memory space
       - Verify       is necessary if Flash space is not mapped into CPU memory space
       - if EraseChip is not provided than EraseSector for all sectors is called
*/


/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {
  return (0);                                  // Finished without Errors
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {
  return (0);                                  // Finished without Errors
}


/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseChip (void) {
  /* Prepare to issue flash operation */
	maxim_flash_op_pre();
  
  /* Write page erase code */
  flsh_cn  = FLASH->FLSH_CN;
  flsh_cn &= 0xFFFF00FF;
  flsh_cn |= 0x0000AA00;
  FLASH->FLSH_CN = flsh_cn;

  /* Issue mass erase command */
  flsh_cn |= 0x2;
  FLASH->FLSH_CN = flsh_cn;

  /* Wait until erase complete */
  while (FLASH->FLSH_CN & FLSH_CN_PEND);
  
  maxim_flash_op_post();
  /* Check access violations */
  if (FLASH->FLSH_INT & FLSH_INT_AF) {
    FLASH->FLSH_INT = 0;
    return (1);
  }
  return (0);                                  // Finished without Errors
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseSector (unsigned long adr) {
  adr &= ~(FLSH_SECTOR_SZ-1);

  /* Prepare to issue flash operation */
	maxim_flash_op_pre();

  /* Address is first word in page. FLSH_ADDR[19:13] are valid. */
	FLASH->FLSH_ADDR  = adr;
  
  /* Write page erase code */
  flsh_cn  = FLASH->FLSH_CN;
  flsh_cn &= 0xFFFF00FF;
  flsh_cn |= 0x00005500;
  FLASH->FLSH_CN = flsh_cn;
  
  /* Issue page erase command */
  flsh_cn |= 0x4;
  FLASH->FLSH_CN = flsh_cn;
  
  /* Wait until erase complete */
  while (FLASH->FLSH_CN & FLSH_CN_PEND);

  
  /* Check access violations */
  if (FLASH->FLSH_INT & FLSH_INT_AF) {
    FLASH->FLSH_INT = 0;
    maxim_flash_op_post();
    return (1);
  }
  maxim_flash_op_post();
  return (0);                                  // Finished without Errors
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {

  /* Prepare to issue flash operation */
	maxim_flash_op_pre();
  
  // write in 32-bit units until we are 128-bit aligned
  flsh_cn    = FLASH->FLSH_CN;
  flsh_cn   &= 0xF7FFFFFF;
  flsh_cn   |= 0x00000010;
  FLASH->FLSH_CN = flsh_cn;
  
  while ( (sz >= 4) && ((adr & 0x1F) != 0) ) {
    FLASH->FLSH_ADDR  = adr;
    FLASH->FLSH_DATA0 = *((u32 *)buf);
    flsh_cn          |= 0x00000001;
    FLASH->FLSH_CN    = flsh_cn;
  
    /* Wait until flash operation is complete */
    while (FLASH->FLSH_CN & FLSH_CN_PEND);
  
    buf += 4;
    adr += 4;
    sz  -= 4;
  }
  
  if (sz >= 16) {
    // write in 128-bit bursts while we can
    flsh_cn          = FLASH->FLSH_CN;
    flsh_cn         &= 0xFFFFFFEF;
    flsh_cn         |= 0x08000000;
    FLASH->FLSH_CN   = flsh_cn;
    FLASH->FLSH_ADDR = adr;
    
    while (sz >= 16) {
      FLASH->FLSH_DATA0 = *((u32 *)buf);
      FLASH->FLSH_DATA1 = *((u32 *)(buf+0x4));
      FLASH->FLSH_DATA2 = *((u32 *)(buf+0x8));
      FLASH->FLSH_DATA3 = *((u32 *)(buf+0xC));
      flsh_cn          |= 0x00000001;
      FLASH->FLSH_CN    = flsh_cn;
      /* Wait until flash operation is complete */
      while (FLASH->FLSH_CN & FLSH_CN_PEND);
      buf += 16;
      adr += 16;
      sz  -= 16;
    }
  }
  
  if (sz >= 4) {
    // write in 32-bit units while we can
    flsh_cn  = FLASH->FLSH_CN;
    flsh_cn &= 0xF7FFFFFF;
    flsh_cn |= 0x00000010;
    FLASH->FLSH_CN = flsh_cn;
    
    while (sz >= 4) {
      FLASH->FLSH_ADDR  = adr;
      FLASH->FLSH_DATA0 = *((u32 *)buf);
      flsh_cn |= 0x00000001;
      FLASH->FLSH_CN = flsh_cn;
    
      /* Wait until flash operation is complete */
      while (FLASH->FLSH_CN & FLSH_CN_PEND);
    
      buf   += 4;
      adr   += 4;
      sz    -= 4;
    }
  }
  
  if (sz > 0) {
    // write remaining bytes in a 32-bit unit
    unsigned char last_word[4] = {0xff, 0xff, 0xff, 0xff};
    int i = 0;
    
    flsh_cn  = FLASH->FLSH_CN;
    flsh_cn &= 0xF7FFFFFF;
    flsh_cn |= 0x00000010;
    FLASH->FLSH_CN = flsh_cn;
  
    while (sz > 0) {
      last_word[i++] = *buf;
      buf++;
      sz--;
    }
    FLASH->FLSH_ADDR = adr;
    FLASH->FLSH_DATA0 = *((u32 *)last_word);
    flsh_cn |= 0x00000001;
    FLASH->FLSH_CN = flsh_cn;
  
  /* Wait until flash operation is complete */
    while (FLASH->FLSH_CN & FLSH_CN_PEND);
  } 
  
  /* Check access violations */
  if (FLASH->FLSH_INT & FLSH_INT_AF) {
    maxim_flash_op_post();
    return (1);
  }
  maxim_flash_op_post();
  return (0);                                  // Finished without Errors
}
